@@ -7,6 +7,7 @@ import os
 import re
 import subprocess
 import sys
+import time
 
 
 def decode_string(encoded: bytes) -> str:
@@ -31,8 +32,10 @@ def test(script_file: str):
         cmd = ["pwsh", "-Command", script_file, ";", "exit $LASTEXITCODE"]
     else:
         cmd = [script_file]
-    print(" ".join(cmd))
+    start_time = time.time()
     result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    execution_time = round(time.time() - start_time, 4)
+    print(f"{' '.join(cmd)} (Duration: {execution_time} seconds)")
 
     basename = os.path.splitext(script_file)[0]
 
