@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,17 +15,18 @@
  * limitations under the License.
  *
  */
+use std::fmt::Display;
+
 use crate::ast::{
     Assert, Base64, Body, BooleanOption, Bytes, Capture, CertificateAttributeName, Comment, Cookie,
     CookieAttribute, CookiePath, CountOption, DurationOption, Entry, EntryOption, File, FileParam,
-    FileValue, Filter, FilterValue, GraphQl, GraphQlVariables, Hex, HurlFile, JsonValue, KeyValue,
-    LineTerminator, Method, MultilineString, MultilineStringKind, MultipartParam, NaturalOption,
-    OptionKind, Placeholder, Predicate, PredicateFunc, PredicateFuncValue, PredicateValue, Query,
-    QueryValue, Regex, RegexValue, Request, Response, Section, SectionValue, Status, Template,
-    TemplateElement, VariableDefinition, VariableValue, Version, Whitespace,
+    FileValue, Filter, FilterValue, Hex, HurlFile, JsonValue, KeyValue, LineTerminator, Method,
+    MultilineString, MultipartParam, NaturalOption, OptionKind, Placeholder, Predicate,
+    PredicateFunc, PredicateFuncValue, PredicateValue, Query, QueryValue, Regex, RegexValue,
+    Request, Response, Section, SectionValue, Status, Template, VariableDefinition, VariableValue,
+    Version, Whitespace,
 };
-use crate::typing::Count;
-use std::fmt::Display;
+use crate::typing::{Count, ToSource};
 
 /// Returns an HTML string of the Hurl file `hurl_file`.
 ///
@@ -126,7 +127,7 @@ impl HtmlFormatter {
         self.fmt_space(&request.space0);
         self.fmt_method(&request.method);
         self.fmt_space(&request.space1);
-        let url = escape_xml(&request.url.to_encoded_string());
+        let url = escape_xml(request.url.to_source().as_str());
         self.fmt_span("url", &url);
         self.fmt_span_close();
         self.fmt_lt(&request.line_terminator0);
@@ -172,7 +173,7 @@ impl HtmlFormatter {
         self.fmt_lts(&section.line_terminators);
         self.fmt_space(&section.space0);
         self.fmt_span_open("line");
-        let name = format!("[{}]", section.name());
+        let name = format!("[{}]", section.identifier());
         self.fmt_span("section-header", &name);
         self.fmt_span_close();
         self.fmt_lt(&section.line_terminator0);
@@ -217,7 +218,7 @@ impl HtmlFormatter {
         self.fmt_lts(&option.line_terminators);
         self.fmt_span_open("line");
         self.fmt_space(&option.space0);
-        self.fmt_string(option.kind.name());
+        self.fmt_string(option.kind.identifier());
         self.fmt_space(&option.space1);
         self.buffer.push(':');
         self.fmt_space(&option.space2);
@@ -232,6 +233,7 @@ impl HtmlFormatter {
             OptionKind::Delay(value) => self.fmt_duration_option(value),
             OptionKind::FollowLocation(value) => self.fmt_bool_option(value),
             OptionKind::FollowLocationTrusted(value) => self.fmt_bool_option(value),
+            OptionKind::Header(value) => self.fmt_template(value),
             OptionKind::Http10(value) => self.fmt_bool_option(value),
             OptionKind::Http11(value) => self.fmt_bool_option(value),
             OptionKind::Http2(value) => self.fmt_bool_option(value),
@@ -288,7 +290,7 @@ impl HtmlFormatter {
         match option {
             VariableValue::Null => self.fmt_span("null", "null"),
             VariableValue::Bool(v) => self.fmt_bool(*v),
-            VariableValue::Number(v) => self.fmt_number(v),
+            VariableValue::Number(v) => self.fmt_number(v.to_source()),
             VariableValue::String(t) => self.fmt_template(t),
         }
     }
@@ -321,7 +323,7 @@ impl HtmlFormatter {
         self.buffer.push(';');
         self.fmt_space(&file_value.space2);
         if let Some(content_type) = &file_value.content_type {
-            self.fmt_string(content_type);
+            self.fmt_template(content_type);
         }
     }
 
@@ -358,6 +360,10 @@ impl HtmlFormatter {
             self.fmt_space(space);
             self.fmt_filter(filter);
         }
+        if capture.redact {
+            self.fmt_space(&capture.space3);
+            self.fmt_string("redact");
+        }
         self.fmt_span_close();
         self.fmt_lt(&capture.line_terminator0);
     }
@@ -367,52 +373,49 @@ impl HtmlFormatter {
     }
 
     fn fmt_query_value(&mut self, query_value: &QueryValue) {
+        let query_type = query_value.identifier();
+        self.fmt_span("query-type", query_type);
         match query_value {
-            QueryValue::Status => self.fmt_span("query-type", "status"),
-            QueryValue::Url => self.fmt_span("query-type", "url"),
             QueryValue::Header { space0, name } => {
-                self.fmt_span("query-type", "header");
                 self.fmt_space(space0);
                 self.fmt_template(name);
             }
             QueryValue::Cookie { space0, expr } => {
-                self.fmt_span("query-type", "cookie");
                 self.fmt_space(space0);
                 self.fmt_cookie_path(expr);
             }
-            QueryValue::Body => self.fmt_span("query-type", "body"),
             QueryValue::Xpath { space0, expr } => {
-                self.fmt_span("query-type", "xpath");
                 self.fmt_space(space0);
                 self.fmt_template(expr);
             }
             QueryValue::Jsonpath { space0, expr } => {
-                self.fmt_span("query-type", "jsonpath");
                 self.fmt_space(space0);
                 self.fmt_template(expr);
             }
             QueryValue::Regex { space0, value } => {
-                self.fmt_span("query-type", "regex");
                 self.fmt_space(space0);
                 self.fmt_regex_value(value);
             }
             QueryValue::Variable { space0, name } => {
-                self.fmt_span("query-type", "variable");
                 self.fmt_space(space0);
                 self.fmt_template(name);
             }
-            QueryValue::Duration => self.fmt_span("query-type", "duration"),
-            QueryValue::Bytes => self.fmt_span("query-type", "bytes"),
-            QueryValue::Sha256 => self.fmt_span("query-type", "sha256"),
-            QueryValue::Md5 => self.fmt_span("query-type", "md5"),
             QueryValue::Certificate {
                 space0,
                 attribute_name: field,
             } => {
-                self.fmt_span("query-type", "certificate");
                 self.fmt_space(space0);
                 self.fmt_certificate_attribute_name(field);
             }
+            QueryValue::Status
+            | QueryValue::Version
+            | QueryValue::Url
+            | QueryValue::Body
+            | QueryValue::Duration
+            | QueryValue::Bytes
+            | QueryValue::Sha256
+            | QueryValue::Md5
+            | QueryValue::Ip => {}
         }
     }
 
@@ -426,8 +429,7 @@ impl HtmlFormatter {
     fn fmt_cookie_path(&mut self, cookie_path: &CookiePath) {
         self.fmt_span_open("string");
         self.buffer.push('"');
-        self.buffer
-            .push_str(cookie_path.name.to_encoded_string().as_str());
+        self.buffer.push_str(cookie_path.name.to_source().as_str());
         if let Some(attribute) = &cookie_path.attribute {
             self.buffer.push('[');
             self.fmt_cookie_attribute(attribute);
@@ -444,16 +446,9 @@ impl HtmlFormatter {
     }
 
     fn fmt_certificate_attribute_name(&mut self, name: &CertificateAttributeName) {
-        let value = match name {
-            CertificateAttributeName::Subject => "Subject",
-            CertificateAttributeName::Issuer => "Issuer",
-            CertificateAttributeName::StartDate => "Start-Date",
-            CertificateAttributeName::ExpireDate => "Expire-Date",
-            CertificateAttributeName::SerialNumber => "Serial-Number",
-        };
         self.fmt_span_open("string");
         self.buffer.push('"');
-        self.buffer.push_str(value);
+        self.buffer.push_str(name.identifier());
         self.buffer.push('"');
         self.fmt_span_close();
     }
@@ -487,7 +482,7 @@ impl HtmlFormatter {
 
     fn fmt_predicate_func_value(&mut self, value: &PredicateFuncValue) {
         self.fmt_span_open("predicate-type");
-        self.buffer.push_str(&encode_html(value.name()));
+        self.buffer.push_str(&encode_html(value.identifier()));
         self.fmt_span_close();
 
         match value {
@@ -545,6 +540,8 @@ impl HtmlFormatter {
             PredicateFuncValue::Exist => {}
             PredicateFuncValue::IsEmpty => {}
             PredicateFuncValue::IsNumber => {}
+            PredicateFuncValue::IsIpv4 => {}
+            PredicateFuncValue::IsIpv6 => {}
         }
     }
 
@@ -552,7 +549,7 @@ impl HtmlFormatter {
         match predicate_value {
             PredicateValue::String(value) => self.fmt_template(value),
             PredicateValue::MultilineString(value) => self.fmt_multiline_string(value, false),
-            PredicateValue::Number(value) => self.fmt_number(value),
+            PredicateValue::Number(value) => self.fmt_number(value.to_source()),
             PredicateValue::Bool(value) => self.fmt_bool(*value),
             PredicateValue::File(value) => self.fmt_file(value),
             PredicateValue::Hex(value) => self.fmt_hex(value),
@@ -564,87 +561,35 @@ impl HtmlFormatter {
     }
 
     fn fmt_multiline_string(&mut self, multiline_string: &MultilineString, as_body: bool) {
-        // The multiline spans multiple newlines. We distinguish cases for multiline
-        // as a body and multiline as a predicate value. When used as a body, we can embed
-        // span lines with the multiline span. Used as a predicate, we have to break the multiline
-        // span in two parts.
-        //
-        // # Case 1: multiline string as a body
-        //
-        // ~~~hurl
-        // GET https://foo.com
-        // ```
-        // line1
-        // line2
-        // line3
-        // ```
-        // ~~~
-        //
-        // We embed span lines inside the span for the body:
-        //
-        // ```html
-        // ...
-        // <span class="multiline">
-        //   <span class="line">```</span>
-        //   <span class="line">line1</span>
-        //   <span class="line">line2</span>
-        //   <span class="line">line3</span>
-        //   <span class="line">```</span>
-        // </span>
-        // ```
-        //
-        // # Case 1: multiline string as a predicate value
-        //
-        // ~~~hurl
-        // GET https://foo.com
-        // HTTP 200
-        // [Asserts]
-        // body == ```
-        // line1
-        // line2
-        // line3
-        // ```
-        // ~~~
-        //
-        // ```html
-        // ...
-        // <span class="line">body ==
-        //   <span class="multiline">```</span>
-        // </span>
-        // <span class="multiline">
-        //   <span class="line">line1</span>
-        //   <span class="line">line2</span>
-        //   <span class="line">line3</span>
-        //   <span class="line">```</span>
-        // </span>
-        // ```
-        let lang = multiline_string.lang();
-        if as_body {
-            let mut attributes = String::new();
-            for (i, attribute) in multiline_string.attributes.iter().enumerate() {
-                if i > 0 || !lang.is_empty() {
-                    attributes.push_str(", ");
-                }
-                attributes.push_str(attribute.to_string().as_str());
-            }
-            // Keep original encoded string
-            let multiline_string_encoded = multiline_string.kind.to_encoded_string();
-            let body = format!("```{lang}{attributes}\n{multiline_string_encoded}```");
-            let body = format_multilines(&body);
-            self.fmt_span("multiline", &body);
-        } else {
-            let head = format!("```{lang}");
-            self.fmt_span("multiline", &head);
-            // We close the current span line opened by the assert
-            self.fmt_span_close();
-            self.buffer.push('\n');
-            let tail = format!("{multiline_string}```");
-            let tail = format_multilines(&tail);
-            self.fmt_span("multiline", &tail);
-            // As we have added a span close, we must remove one to have the right number
-            // of span. The current span line will add a closing span.
-            pop_str(&mut self.buffer, "</span>");
+        let body = multiline_string.to_source();
+        let mut body = format_lines(body.as_str(), true);
+        if !as_body {
+            // A multiline AST element spans multiple line. When used as an assert, the AST element in
+            // in the middle of the current line:
+            //
+            // ~~~
+            // GET https://foo.com
+            // HTTP 200
+            // [Asserts]
+            // body == ```
+            // line1
+            // line2
+            // ```
+            // ~~~
+            //
+            // We don't want the multiline AST to begin a new `<span clas="line">`
+            // element so we split the multiline AST element in a list of single-line "multiline"
+            // elements. This way, each new multiline element is wrapped in a single line.
+            // NOTE: this still feels hacky to me, I'm not sure that we should add span for lines, it
+            // intermixes an AST hierarchical view and a line oriented HTML view.
+            body = body
+                .strip_prefix("<span class=\"line\">")
+                .unwrap()
+                .strip_suffix("</span>")
+                .unwrap()
+                .to_string();
         }
+        self.buffer.push_str(&body);
     }
 
     fn fmt_body(&mut self, body: &Body) {
@@ -699,16 +644,18 @@ impl HtmlFormatter {
 
     fn fmt_natural_option(&mut self, value: &NaturalOption) {
         match value {
-            NaturalOption::Literal(value) => self.fmt_span("number", &value.to_string()),
+            NaturalOption::Literal(value) => {
+                self.fmt_span("number", &value.to_source().to_string());
+            }
             NaturalOption::Placeholder(value) => self.fmt_placeholder(value),
         }
     }
 
     fn fmt_duration_option(&mut self, value: &DurationOption) {
         match value {
-            DurationOption::Literal(value) => {
-                self.fmt_span("number", &value.value.to_string());
-                if let Some(unit) = value.unit {
+            DurationOption::Literal(literal) => {
+                self.fmt_span("number", &literal.value.to_source().to_string());
+                if let Some(unit) = literal.unit {
                     self.fmt_span("unit", &unit.to_string());
                 }
             }
@@ -721,12 +668,12 @@ impl HtmlFormatter {
     }
 
     fn fmt_xml(&mut self, value: &str) {
-        let xml = format_multilines(value);
+        let xml = format_lines(value, false);
         self.fmt_span("xml", &xml);
     }
 
     fn fmt_json_value(&mut self, json_value: &JsonValue) {
-        let json = format_multilines(&json_value.encoded());
+        let json = format_lines(json_value.to_source().as_str(), false);
         self.fmt_span("json", &json);
     }
 
@@ -742,7 +689,7 @@ impl HtmlFormatter {
         if let Some(v) = &lt.comment {
             self.fmt_comment(v);
         }
-        self.buffer.push_str(lt.newline.value.as_str());
+        self.buffer.push_str(lt.newline.as_str());
     }
 
     fn fmt_comment(&mut self, comment: &Comment) {
@@ -761,7 +708,7 @@ impl HtmlFormatter {
     fn fmt_base64(&mut self, base64: &Base64) {
         self.buffer.push_str("base64,");
         self.fmt_space(&base64.space0);
-        self.fmt_span("base64", &base64.encoded);
+        self.fmt_span("base64", &base64.source.to_string());
         self.fmt_space(&base64.space1);
         self.buffer.push(';');
     }
@@ -769,7 +716,7 @@ impl HtmlFormatter {
     fn fmt_hex(&mut self, hex: &Hex) {
         self.buffer.push_str("hex,");
         self.fmt_space(&hex.space0);
-        self.fmt_span("hex", &hex.encoded);
+        self.fmt_span("hex", &hex.source.to_string());
         self.fmt_space(&hex.space1);
         self.buffer.push(';');
     }
@@ -781,13 +728,13 @@ impl HtmlFormatter {
     }
 
     fn fmt_template(&mut self, template: &Template) {
-        let s = template.to_encoded_string();
-        self.fmt_string(&escape_xml(&s));
+        let s = template.to_source();
+        self.fmt_string(&escape_xml(s.as_str()));
     }
 
     fn fmt_placeholder(&mut self, placeholder: &Placeholder) {
-        let placeholder = format!("{{{{{}}}}}", &placeholder.to_string());
-        self.fmt_span("expr", &placeholder);
+        let placeholder = placeholder.to_source();
+        self.fmt_span("expr", placeholder.as_str());
     }
 
     fn fmt_filter(&mut self, filter: &Filter) {
@@ -795,34 +742,25 @@ impl HtmlFormatter {
     }
 
     fn fmt_filter_value(&mut self, filter_value: &FilterValue) {
+        self.fmt_span("filter-type", filter_value.identifier());
         match filter_value {
-            FilterValue::Count => self.fmt_span("filter-type", "count"),
-            FilterValue::DaysAfterNow => self.fmt_span("filter-type", "daysAfterNow"),
-            FilterValue::DaysBeforeNow => self.fmt_span("filter-type", "daysBeforeNow"),
             FilterValue::Decode { space0, encoding } => {
-                self.fmt_span("filter-type", "decode");
                 self.fmt_space(space0);
                 self.fmt_template(encoding);
             }
             FilterValue::Format { space0, fmt } => {
-                self.fmt_span("filter-type", "format");
                 self.fmt_space(space0);
                 self.fmt_template(fmt);
             }
-            FilterValue::HtmlEscape => self.fmt_span("filter-type", "htmlEscape"),
-            FilterValue::HtmlUnescape => self.fmt_span("filter-type", "htmlUnescape"),
             FilterValue::JsonPath { space0, expr } => {
-                self.fmt_span("filter-type", "jsonpath");
                 self.fmt_space(space0);
                 self.fmt_template(expr);
             }
             FilterValue::Nth { space0, n: value } => {
-                self.fmt_span("filter-type", "nth");
                 self.fmt_space(space0);
-                self.fmt_number(value);
+                self.fmt_number(value.to_source());
             }
             FilterValue::Regex { space0, value } => {
-                self.fmt_span("filter-type", "regex");
                 self.fmt_space(space0);
                 self.fmt_regex_value(value);
             }
@@ -832,31 +770,35 @@ impl HtmlFormatter {
                 space1,
                 new_value,
             } => {
-                self.fmt_span("filter-type", "replace");
                 self.fmt_space(space0);
                 self.fmt_regex_value(old_value);
                 self.fmt_space(space1);
                 self.fmt_template(new_value);
             }
             FilterValue::Split { space0, sep } => {
-                self.fmt_span("filter-type", "split");
                 self.fmt_space(space0);
                 self.fmt_template(sep);
             }
             FilterValue::ToDate { space0, fmt } => {
-                self.fmt_span("filter-type", "toDate");
                 self.fmt_space(space0);
                 self.fmt_template(fmt);
             }
-            FilterValue::ToFloat => self.fmt_span("filter-type", "toFloat"),
-            FilterValue::ToInt => self.fmt_span("filter-type", "toInt"),
-            FilterValue::UrlDecode => self.fmt_span("filter-type", "urlDecode"),
-            FilterValue::UrlEncode => self.fmt_span("filter-type", "urlEncode"),
             FilterValue::XPath { space0, expr } => {
-                self.fmt_span("filter-type", "xpath");
                 self.fmt_space(space0);
                 self.fmt_template(expr);
             }
+            FilterValue::Base64Decode
+            | FilterValue::Base64Encode
+            | FilterValue::Count
+            | FilterValue::DaysAfterNow
+            | FilterValue::DaysBeforeNow
+            | FilterValue::HtmlEscape
+            | FilterValue::HtmlUnescape
+            | FilterValue::ToFloat
+            | FilterValue::ToInt
+            | FilterValue::ToString
+            | FilterValue::UrlDecode
+            | FilterValue::UrlEncode => {}
         };
     }
 
@@ -878,82 +820,32 @@ fn escape_xml(s: &str) -> String {
         .replace('>', "&gt;")
 }
 
-impl Template {
-    fn to_encoded_string(&self) -> String {
-        let mut s = String::new();
-        if let Some(d) = self.delimiter {
-            s.push(d);
-        }
-        for element in self.elements.iter() {
-            let elem_str = match element {
-                TemplateElement::String { encoded, .. } => encoded.to_string(),
-                TemplateElement::Placeholder(expr) => format!("{{{{{expr}}}}}"),
-            };
-            s.push_str(elem_str.as_str());
-        }
-        if let Some(d) = self.delimiter {
-            s.push(d);
-        }
-        s
-    }
-}
-
-impl MultilineStringKind {
-    fn to_encoded_string(&self) -> String {
-        match self {
-            MultilineStringKind::Text(text)
-            | MultilineStringKind::Json(text)
-            | MultilineStringKind::Xml(text) => text.value.to_encoded_string(),
-            MultilineStringKind::GraphQl(graphql) => graphql.to_encoded_string(),
-        }
-    }
-}
-
-impl GraphQl {
-    fn to_encoded_string(&self) -> String {
-        let mut s = self.value.to_encoded_string();
-        if let Some(vars) = &self.variables {
-            s.push_str(&vars.to_encoded_string());
-        }
-        s
-    }
-}
-
-impl GraphQlVariables {
-    fn to_encoded_string(&self) -> String {
-        let mut s = "variables".to_string();
-        s.push_str(&self.space.value);
-        s.push_str(&self.value.encoded());
-        s.push_str(&self.whitespace.value);
-        s
-    }
-}
-
-fn encode_html(s: String) -> String {
+fn encode_html(s: &str) -> String {
     s.replace('>', "&gt;").replace('<', "&lt;")
 }
 
-fn format_multilines(s: &str) -> String {
+fn format_lines(s: &str, use_multiline_class: bool) -> String {
     regex::Regex::new(r"\n|\r\n")
         .unwrap()
         .split(s)
-        .map(|l| format!("<span class=\"line\">{}</span>", escape_xml(l)))
+        .map(|l| {
+            let text = escape_xml(l);
+            if use_multiline_class {
+                format!("<span class=\"line\"><span class=\"multiline\">{text}</span></span>")
+            } else {
+                format!("<span class=\"line\">{text}</span>")
+            }
+        })
         .collect::<Vec<String>>()
         .join("\n")
 }
 
-fn pop_str(string: &mut String, suffix: &str) {
-    let len = string.len();
-    let n = suffix.len();
-    let len = len - n;
-    string.truncate(len);
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::ast::{JsonObjectElement, SourceInfo, Text};
+    use crate::ast::{JsonObjectElement, MultilineStringKind, SourceInfo, TemplateElement};
     use crate::reader::Pos;
+    use crate::typing::ToSource;
 
     #[test]
     fn test_multiline_string() {
@@ -961,7 +853,17 @@ mod tests {
         // line1
         // line2
         // ```
-        let kind = MultilineStringKind::Text(Text {
+        let kind = MultilineStringKind::Text(Template {
+            delimiter: None,
+            elements: vec![TemplateElement::String {
+                value: "line1\nline2\n".to_string(),
+                source: "line1\nline2\n".to_source(),
+            }],
+            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+        });
+        let attributes = vec![];
+        let multiline_string = MultilineString {
+            attributes,
             space: Whitespace {
                 value: String::new(),
                 source_info: SourceInfo {
@@ -976,26 +878,23 @@ mod tests {
                     end: Pos { line: 2, column: 1 },
                 },
             },
-            value: Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
-                    value: "line1\nline2\n".to_string(),
-                    encoded: "line1\nline2\n".to_string(),
-                }],
-                source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-            },
-        });
-        let attributes = vec![];
-        let multiline_string = MultilineString { kind, attributes };
+            kind,
+        };
         let mut fmt = HtmlFormatter::new();
         fmt.fmt_multiline_string(&multiline_string, true);
         assert_eq!(
             fmt.buffer,
-            "<span class=\"multiline\">\
-                <span class=\"line\">```</span>\n\
-                <span class=\"line\">line1</span>\n\
-                <span class=\"line\">line2</span>\n\
-                <span class=\"line\">```</span>\
+            "<span class=\"line\">\
+                <span class=\"multiline\">```</span>\
+            </span>\n\
+            <span class=\"line\">\
+                <span class=\"multiline\">line1</span>\
+            </span>\n\
+            <span class=\"line\">\
+                <span class=\"multiline\">line2</span>\
+            </span>\n\
+            <span class=\"line\">\
+                <span class=\"multiline\">```</span>\
             </span>"
         );
 
@@ -1005,32 +904,44 @@ mod tests {
             fmt.buffer,
             "<span class=\"multiline\">```</span>\
         </span>\n\
-        <span class=\"multiline\">\
-            <span class=\"line\">line1</span>\n\
-            <span class=\"line\">line2</span>\n\
-            <span class=\"line\">```</span>"
+        <span class=\"line\">\
+            <span class=\"multiline\">line1</span>\
+        </span>\n\
+        <span class=\"line\">\
+            <span class=\"multiline\">line2</span>\
+        </span>\n\
+        <span class=\"line\">\
+            <span class=\"multiline\">```</span>"
         );
     }
 
     #[test]
     fn test_multilines() {
         assert_eq!(
-            format_multilines("{\n   \"id\": 1\n}"),
+            format_lines("{\n   \"id\": 1\n}", false),
             "<span class=\"line\">{</span>\n\
             <span class=\"line\">   \"id\": 1</span>\n\
             <span class=\"line\">}</span>"
         );
         assert_eq!(
-            format_multilines(
+            format_lines("{\n   \"id\": 1\n}", true),
+            "<span class=\"line\"><span class=\"multiline\">{</span></span>\n\
+            <span class=\"line\"><span class=\"multiline\">   \"id\": 1</span></span>\n\
+            <span class=\"line\"><span class=\"multiline\">}</span></span>"
+        );
+
+        assert_eq!(
+            format_lines(
                 "<?xml version=\"1.0\"?>\n\
-            <drink>café</drink>"
+            <drink>café</drink>",
+                false
             ),
             "<span class=\"line\">&lt;?xml version=\"1.0\"?&gt;</span>\n\
             <span class=\"line\">&lt;drink&gt;café&lt;/drink&gt;</span>"
         );
 
         assert_eq!(
-            format_multilines("Hello\n"),
+            format_lines("Hello\n", false),
             "<span class=\"line\">Hello</span>\n\
             <span class=\"line\"></span>"
         );
@@ -1043,14 +954,14 @@ mod tests {
             space0: String::new(),
             elements: vec![JsonObjectElement {
                 space0: "\n   ".to_string(),
-                name: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                name: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "id".to_string(),
-                        encoded: "id".to_string(),
+                        source: "id".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-                },
+                    SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+                ),
                 space1: String::new(),
                 space2: " ".to_string(),
                 value: JsonValue::Number("1".to_string()),
@@ -1071,14 +982,14 @@ mod tests {
     #[test]
     fn test_json_encoded_newline() {
         let mut fmt = HtmlFormatter::new();
-        let value = JsonValue::String(Template {
-            delimiter: Some('"'),
-            elements: vec![TemplateElement::String {
+        let value = JsonValue::String(Template::new(
+            Some('"'),
+            vec![TemplateElement::String {
                 value: "\n".to_string(),
-                encoded: "\\n".to_string(),
+                source: "\\n".to_source(),
             }],
-            source_info: SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
-        });
+            SourceInfo::new(Pos::new(0, 0), Pos::new(0, 0)),
+        ));
         fmt.fmt_json_value(&value);
         assert_eq!(
             fmt.buffer,
