@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,12 +15,12 @@
  * limitations under the License.
  *
  */
+use super::placeholder;
 use crate::ast::{SourceInfo, Template, TemplateElement};
 use crate::parser::primitives::try_literal;
 use crate::parser::{string, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::Reader;
-
-use super::placeholder;
+use crate::typing::ToSource;
 
 /// Parse a filename.
 ///
@@ -45,8 +45,8 @@ pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
                     if value.is_empty() {
                         break;
                     }
-                    let encoded = reader.read_from(start.index);
-                    let element = TemplateElement::String { value, encoded };
+                    let source = reader.read_from(start.index).to_source();
+                    let element = TemplateElement::String { value, source };
                     elements.push(element);
                 } else {
                     return Err(e);
@@ -58,8 +58,8 @@ pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
         let kind = ParseErrorKind::Filename;
         return Err(ParseError::new(start.pos, false, kind));
     }
-    if let Some(TemplateElement::String { encoded, .. }) = elements.first() {
-        if encoded.starts_with('[') {
+    if let Some(TemplateElement::String { source, .. }) = elements.first() {
+        if source.starts_with('[') {
             let kind = ParseErrorKind::Expecting {
                 value: "filename".to_string(),
             };
@@ -68,14 +68,8 @@ pub fn parse(reader: &mut Reader) -> ParseResult<Template> {
     }
 
     let end = reader.cursor();
-    Ok(Template {
-        delimiter: None,
-        elements,
-        source_info: SourceInfo {
-            start: start.pos,
-            end: end.pos,
-        },
-    })
+    let template = Template::new(None, elements, SourceInfo::new(start.pos, end.pos));
+    Ok(template)
 }
 
 fn filename_content(reader: &mut Reader) -> ParseResult<String> {
@@ -156,29 +150,28 @@ mod tests {
         let mut reader = Reader::new("data/data.bin");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "data/data.bin".to_string(),
-                    encoded: "data/data.bin".to_string()
+                    source: "data/data.bin".to_source()
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 14)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 14)),
+            )
         );
         assert_eq!(reader.cursor().index, 13);
 
         let mut reader = Reader::new("data.bin");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                //value: String::from("data.bin"),
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "data.bin".to_string(),
-                    encoded: "data.bin".to_string()
+                    source: "data.bin".to_source()
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 9)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 9)),
+            )
         );
         assert_eq!(reader.cursor().index, 8);
     }
@@ -188,14 +181,14 @@ mod tests {
         let mut reader = Reader::new("file\\ with\\ spaces");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "file with spaces".to_string(),
-                    encoded: "file\\ with\\ spaces".to_string()
+                    source: "file\\ with\\ spaces".to_source()
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 19)),
+            )
         );
         assert_eq!(reader.cursor().index, 18);
     }
@@ -205,14 +198,14 @@ mod tests {
         let mut reader = Reader::new("filename\\{"); // to the possible escaped chars
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "filename{".to_string(),
-                    encoded: "filename\\{".to_string()
+                    source: "filename\\{".to_source()
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 11)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 11))
+            )
         );
         assert_eq!(reader.cursor().index, 10);
     }
@@ -235,12 +228,12 @@ mod tests {
         let mut reader = Reader::new("foo_{{bar}}");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![
+            Template::new(
+                None,
+                vec![
                     TemplateElement::String {
                         value: "foo_".to_string(),
-                        encoded: "foo_".to_string(),
+                        source: "foo_".to_source(),
                     },
                     TemplateElement::Placeholder(Placeholder {
                         space0: Whitespace {
@@ -260,19 +253,19 @@ mod tests {
                         },
                     })
                 ],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 12)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 12)),
+            )
         );
 
         let mut reader = Reader::new("foo_{{bar}}_baz");
         assert_eq!(
             parse(&mut reader).unwrap(),
-            Template {
-                delimiter: None,
-                elements: vec![
+            Template::new(
+                None,
+                vec![
                     TemplateElement::String {
                         value: "foo_".to_string(),
-                        encoded: "foo_".to_string(),
+                        source: "foo_".to_source(),
                     },
                     TemplateElement::Placeholder(Placeholder {
                         space0: Whitespace {
@@ -293,11 +286,11 @@ mod tests {
                     }),
                     TemplateElement::String {
                         value: "_baz".to_string(),
-                        encoded: "_baz".to_string(),
+                        source: "_baz".to_source(),
                     },
                 ],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 16)),
-            }
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 16)),
+            )
         );
     }
 }
