@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -240,7 +240,14 @@ fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
         Err(_) => {
             reader.seek(save);
             let space2 = zero_or_more_spaces(reader)?;
-            let content_type = file_content_type(reader)?;
+            let start = reader.cursor();
+            let Ok(content_type) = unquoted_template(reader) else {
+                return Err(ParseError::new(
+                    start.pos,
+                    false,
+                    ParseErrorKind::FileContentType,
+                ));
+            };
             (space2, Some(content_type))
         }
     };
@@ -254,35 +261,6 @@ fn file_value(reader: &mut Reader) -> ParseResult<FileValue> {
     })
 }
 
-fn file_content_type(reader: &mut Reader) -> ParseResult<String> {
-    let start = reader.cursor();
-    let mut buf = String::new();
-    let mut spaces = String::new();
-    let mut save = reader.cursor();
-    while let Some(c) = reader.read() {
-        if c.is_alphanumeric() || c == '/' || c == ';' || c == '=' || c == '-' {
-            buf.push_str(spaces.as_str());
-            spaces = String::new();
-            buf.push(c);
-            save = reader.cursor();
-        } else if c == ' ' {
-            spaces.push(' ');
-        } else {
-            break;
-        }
-    }
-
-    reader.seek(save);
-    if buf.is_empty() {
-        return Err(ParseError::new(
-            start.pos,
-            false,
-            ParseErrorKind::FileContentType,
-        ));
-    }
-    Ok(buf)
-}
-
 fn capture(reader: &mut Reader) -> ParseResult<Capture> {
     let line_terminators = optional_line_terminators(reader)?;
     let space0 = zero_or_more_spaces(reader)?;
@@ -292,6 +270,8 @@ fn capture(reader: &mut Reader) -> ParseResult<Capture> {
     let space2 = zero_or_more_spaces(reader)?;
     let q = query(reader)?;
     let filters = filters(reader)?;
+    let space3 = zero_or_more_spaces(reader)?;
+    let redact = try_literal("redact", reader).is_ok();
     let line_terminator0 = line_terminator(reader)?;
     Ok(Capture {
         line_terminators,
@@ -301,6 +281,8 @@ fn capture(reader: &mut Reader) -> ParseResult<Capture> {
         space2,
         query: q,
         filters,
+        space3,
+        redact,
         line_terminator0,
     })
 }
@@ -330,8 +312,9 @@ mod tests {
     use super::*;
     use crate::ast::{
         KeyValue, LineTerminator, Number, Predicate, PredicateFunc, PredicateFuncValue,
-        PredicateValue, Query, QueryValue, Template, TemplateElement,
+        PredicateValue, Query, QueryValue, Template, TemplateElement, I64,
     };
+    use crate::typing::ToSource;
 
     #[test]
     fn test_section_name() {
@@ -378,14 +361,14 @@ mod tests {
                                 value: String::from(" "),
                                 source_info: SourceInfo::new(Pos::new(2, 7), Pos::new(2, 8)),
                             },
-                            name: Template {
-                                delimiter: Some('"'),
-                                elements: vec![TemplateElement::String {
+                            name: Template::new(
+                                Some('"'),
+                                vec![TemplateElement::String {
                                     value: "Location".to_string(),
-                                    encoded: "Location".to_string(),
+                                    source: "Location".to_source(),
                                 }],
-                                source_info: SourceInfo::new(Pos::new(2, 8), Pos::new(2, 18)),
-                            },
+                                SourceInfo::new(Pos::new(2, 8), Pos::new(2, 18)),
+                            ),
                         },
                     },
                     filters: vec![],
@@ -406,15 +389,14 @@ mod tests {
                                     value: String::from(" "),
                                     source_info: SourceInfo::new(Pos::new(2, 21), Pos::new(2, 22)),
                                 },
-                                value: PredicateValue::String(Template {
-                                    delimiter: Some('"'),
-                                    elements: vec![TemplateElement::String {
+                                value: PredicateValue::String(Template::new(
+                                    Some('"'),
+                                    vec![TemplateElement::String {
                                         value: "https://google.fr".to_string(),
-                                        encoded: "https://google.fr".to_string(),
+                                        source: "https://google.fr".to_source(),
                                     }],
-                                    source_info: SourceInfo::new(Pos::new(2, 22), Pos::new(2, 41)),
-                                }),
-                                operator: true,
+                                    SourceInfo::new(Pos::new(2, 22), Pos::new(2, 41))
+                                )),
                             },
                         },
                     },
@@ -467,14 +449,14 @@ mod tests {
         assert_eq!(c.name.to_string(), String::from("Foo"));
         assert_eq!(
             c.value,
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "Bar".to_string(),
-                    encoded: "Bar".to_string(),
+                    source: "Bar".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 9)),
-            }
+                SourceInfo::new(Pos::new(1, 6), Pos::new(1, 9))
+            )
         );
     }
 
@@ -508,14 +490,14 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
                 },
-                filename: Template {
-                    delimiter: None,
-                    elements: vec![TemplateElement::String {
+                filename: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "hello.txt".to_string(),
-                        encoded: "hello.txt".to_string(),
+                        source: "hello.txt".to_source(),
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 15)),
-                },
+                    SourceInfo::new(Pos::new(1, 6), Pos::new(1, 15)),
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
@@ -535,14 +517,14 @@ mod tests {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 6)),
                 },
-                filename: Template {
-                    elements: vec![TemplateElement::String {
+                filename: Template::new(
+                    None,
+                    vec![TemplateElement::String {
                         value: "hello.txt".to_string(),
-                        encoded: "hello.txt".to_string(),
+                        source: "hello.txt".to_source(),
                     }],
-                    delimiter: None,
-                    source_info: SourceInfo::new(Pos::new(1, 6), Pos::new(1, 15)),
-                },
+                    SourceInfo::new(Pos::new(1, 6), Pos::new(1, 15)),
+                ),
                 space1: Whitespace {
                     value: String::new(),
                     source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 15)),
@@ -551,33 +533,58 @@ mod tests {
                     value: " ".to_string(),
                     source_info: SourceInfo::new(Pos::new(1, 16), Pos::new(1, 17)),
                 },
-                content_type: Some("text/html".to_string()),
+                content_type: Some(Template::new(
+                    None,
+                    vec![TemplateElement::String {
+                        value: "text/html".to_string(),
+                        source: "text/html".to_source(),
+                    }],
+                    SourceInfo::new(Pos::new(1, 17), Pos::new(1, 26)),
+                )),
             }
         );
     }
 
     #[test]
     fn test_file_content_type() {
-        let mut reader = Reader::new("text/html");
-        assert_eq!(
-            file_content_type(&mut reader).unwrap(),
-            "text/html".to_string()
-        );
-        assert_eq!(reader.cursor().index, 9);
+        let mut reader = Reader::new("file,hello.txt; text/html");
+        let file_value = file_value(&mut reader).unwrap();
+        let content_type = file_value.content_type.unwrap();
+        assert_eq!(content_type.to_string(), "text/html".to_string());
+        assert_eq!(reader.cursor().index, 25);
 
-        let mut reader = Reader::new("text/plain; charset=us-ascii");
+        let mut reader = Reader::new("file,------; text/plain; charset=us-ascii");
+        let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
+        let content_type = file_value.content_type.unwrap();
         assert_eq!(
-            file_content_type(&mut reader).unwrap(),
+            content_type.to_string(),
             "text/plain; charset=us-ascii".to_string()
         );
-        assert_eq!(reader.cursor().index, 28);
+        assert_eq!(reader.cursor().index, 41);
 
-        let mut reader = Reader::new("text/html # comment");
+        let mut reader = Reader::new("file,******; text/html # comment");
+        let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
+        let content_type = file_value.content_type.unwrap();
+        assert_eq!(content_type.to_string(), "text/html".to_string());
+        assert_eq!(reader.cursor().index, 22);
+
+        let mut reader = Reader::new("file,{{some_file}}; application/vnd.openxmlformats-officedocument.wordprocessingml.document # comment");
+        let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
+        let content_type = file_value.content_type.unwrap();
         assert_eq!(
-            file_content_type(&mut reader).unwrap(),
-            "text/html".to_string()
+            content_type.to_string(),
+            "application/vnd.openxmlformats-officedocument.wordprocessingml.document".to_string()
         );
-        assert_eq!(reader.cursor().index, 9);
+        assert_eq!(reader.cursor().index, 91);
+
+        let mut reader = Reader::new("file,{{some_file}}; {{some_content_type}} # comment");
+        let file_value = crate::parser::sections::file_value(&mut reader).unwrap();
+        let content_type = file_value.content_type.unwrap();
+        assert_eq!(
+            content_type.to_string(),
+            "{{some_content_type}}".to_string()
+        );
+        assert_eq!(reader.cursor().index, 41);
     }
 
     #[test]
@@ -587,14 +594,14 @@ mod tests {
 
         assert_eq!(
             capture0.name,
-            Template {
-                delimiter: None,
-                elements: vec![TemplateElement::String {
+            Template::new(
+                None,
+                vec![TemplateElement::String {
                     value: "url".to_string(),
-                    encoded: "url".to_string(),
+                    source: "url".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4)),
-            },
+                SourceInfo::new(Pos::new(1, 1), Pos::new(1, 4))
+            ),
         );
         assert_eq!(
             capture0.query,
@@ -605,17 +612,21 @@ mod tests {
                         value: String::from(" "),
                         source_info: SourceInfo::new(Pos::new(1, 12), Pos::new(1, 13)),
                     },
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "Location".to_string(),
-                            encoded: "Location".to_string(),
+                            source: "Location".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(1, 13), Pos::new(1, 23)),
-                    },
-                },
+                        SourceInfo::new(Pos::new(1, 13), Pos::new(1, 23))
+                    )
+                }
             }
         );
+
+        let mut reader = Reader::new("url: header \"Token\" redact");
+        let capture0 = capture(&mut reader).unwrap();
+        assert!(capture0.redact);
     }
 
     #[test]
@@ -632,15 +643,15 @@ mod tests {
                         value: String::from(" "),
                         source_info: SourceInfo::new(Pos::new(1, 14), Pos::new(1, 15)),
                     },
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "Location".to_string(),
-                            encoded: "Location".to_string(),
+                            source: "Location".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(1, 15), Pos::new(1, 25)),
-                    },
-                },
+                        SourceInfo::new(Pos::new(1, 15), Pos::new(1, 25))
+                    )
+                }
             }
         );
         assert_eq!(reader.cursor().index, 43);
@@ -697,15 +708,15 @@ mod tests {
                         value: String::from(" "),
                         source_info: SourceInfo::new(Pos::new(1, 7), Pos::new(1, 8)),
                     },
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "Location".to_string(),
-                            encoded: "Location".to_string(),
+                            source: "Location".to_source(),
                         }],
-                        source_info: SourceInfo::new(Pos::new(1, 8), Pos::new(1, 18)),
-                    },
-                },
+                        SourceInfo::new(Pos::new(1, 8), Pos::new(1, 18)),
+                    )
+                }
             }
         );
     }
@@ -729,8 +740,10 @@ mod tests {
                             value: String::from(" "),
                             source_info: SourceInfo::new(Pos::new(1, 23), Pos::new(1, 24)),
                         },
-                        value: PredicateValue::Number(Number::Integer(5)),
-                        operator: true,
+                        value: PredicateValue::Number(Number::Integer(I64::new(
+                            5,
+                            "5".to_source()
+                        ))),
                     },
                 },
             }
@@ -766,14 +779,14 @@ mod tests {
                         value: String::new(),
                         source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 1))
                     },
-                    key: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
+                    key: Template::new(
+                        None,
+                        vec![TemplateElement::String {
                             value: "user".to_string(),
-                            encoded: "user".to_string()
+                            source: "user".to_source()
                         }],
-                        source_info: SourceInfo::new(Pos::new(2, 1), Pos::new(2, 5)),
-                    },
+                        SourceInfo::new(Pos::new(2, 1), Pos::new(2, 5)),
+                    ),
                     space1: Whitespace {
                         value: String::new(),
                         source_info: SourceInfo::new(Pos::new(2, 5), Pos::new(2, 5))
@@ -782,14 +795,14 @@ mod tests {
                         value: String::new(),
                         source_info: SourceInfo::new(Pos::new(2, 6), Pos::new(2, 6))
                     },
-                    value: Template {
-                        delimiter: None,
-                        elements: vec![TemplateElement::String {
+                    value: Template::new(
+                        None,
+                        vec![TemplateElement::String {
                             value: "password".to_string(),
-                            encoded: "password".to_string()
+                            source: "password".to_source()
                         }],
-                        source_info: SourceInfo::new(Pos::new(2, 6), Pos::new(2, 14)),
-                    },
+                        SourceInfo::new(Pos::new(2, 6), Pos::new(2, 14)),
+                    ),
                     line_terminator0: LineTerminator {
                         space0: Whitespace {
                             value: String::new(),
