@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,16 +17,16 @@
  */
 use hurl_core::ast::{
     Assert, Base64, Body, BooleanOption, Bytes, Capture, CertificateAttributeName, Comment, Cookie,
-    CookieAttribute, CookiePath, CountOption, DurationOption, EncodedString, Entry, EntryOption,
-    Expr, ExprKind, File, FileParam, FileValue, Filter, FilterValue, Function, GraphQl,
-    GraphQlVariables, Hex, HurlFile, JsonListElement, JsonObjectElement, JsonValue, KeyValue,
-    LineTerminator, Method, MultilineString, MultilineStringAttribute, MultilineStringKind,
-    MultipartParam, NaturalOption, OptionKind, Placeholder, Predicate, PredicateFunc,
-    PredicateFuncValue, PredicateValue, Query, QueryValue, Regex, RegexValue, Request, Response,
-    Section, SectionValue, Status, StatusValue, Template, TemplateElement, Text, Variable,
-    VariableDefinition, VariableValue, Version, Whitespace,
+    CookieAttribute, CookiePath, CountOption, DurationOption, Entry, EntryOption, Expr, ExprKind,
+    File, FileParam, FileValue, Filter, FilterValue, Function, GraphQl, GraphQlVariables, Hex,
+    HurlFile, JsonListElement, JsonObjectElement, JsonValue, KeyValue, LineTerminator, Method,
+    MultilineString, MultilineStringAttribute, MultilineStringKind, MultipartParam, NaturalOption,
+    OptionKind, Placeholder, Predicate, PredicateFunc, PredicateFuncValue, PredicateValue, Query,
+    QueryValue, Regex, RegexValue, Request, Response, Section, SectionValue, Status, StatusValue,
+    Template, TemplateElement, Variable, VariableDefinition, VariableValue, Version, Whitespace,
+    I64, U64,
 };
-use hurl_core::typing::{Count, Duration};
+use hurl_core::typing::{Count, Duration, ToSource};
 
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum Token {
@@ -200,7 +200,7 @@ impl Tokenizable for Section {
                 .collect(),
         );
         tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::SectionHeader(format!("[{}]", self.name())));
+        tokens.push(Token::SectionHeader(format!("[{}]", self.identifier())));
         tokens.append(&mut self.line_terminator0.tokenize());
         tokens.append(&mut self.value.tokenize());
         tokens
@@ -246,7 +246,7 @@ impl Tokenizable for Base64 {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![Token::Keyword(String::from("base64,"))];
         tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String(self.encoded.to_string()));
+        tokens.push(Token::String(self.source.to_string()));
         tokens.append(&mut self.space1.tokenize());
         tokens.push(Token::Keyword(String::from(";")));
         tokens
@@ -257,7 +257,7 @@ impl Tokenizable for Hex {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![Token::Keyword(String::from("hex,"))];
         tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String(self.encoded.to_string()));
+        tokens.push(Token::String(self.source.to_string()));
         tokens.append(&mut self.space1.tokenize());
         tokens.push(Token::Keyword(String::from(";")));
         tokens
@@ -327,8 +327,8 @@ impl Tokenizable for FileValue {
         tokens.append(&mut self.space1.tokenize());
         tokens.push(Token::Keyword(";".to_string()));
         tokens.append(&mut self.space2.tokenize());
-        if let Some(content_type) = self.content_type.clone() {
-            tokens.push(Token::String(content_type));
+        if let Some(content_type) = &self.content_type {
+            tokens.push(Token::String(content_type.to_string()));
         }
         tokens
     }
@@ -375,6 +375,10 @@ impl Tokenizable for Capture {
             tokens.append(&mut space.tokenize());
             tokens.append(&mut filter.tokenize());
         }
+        if self.redact {
+            tokens.append(&mut self.space3.tokenize());
+            tokens.push(Token::Keyword(String::from("redact")));
+        }
         tokens.append(&mut self.line_terminator0.tokenize());
         tokens
     }
@@ -413,55 +417,45 @@ impl Tokenizable for Query {
 
 impl Tokenizable for QueryValue {
     fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        match self.clone() {
-            QueryValue::Status => tokens.push(Token::QueryType(String::from("status"))),
-            QueryValue::Url => tokens.push(Token::QueryType(String::from("url"))),
+        let mut tokens = vec![];
+        let token = Token::QueryType(self.identifier().to_string());
+        tokens.push(token);
+
+        match self {
             QueryValue::Header { space0, name } => {
-                tokens.push(Token::QueryType(String::from("header")));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut name.tokenize());
             }
             QueryValue::Cookie { space0, expr } => {
-                tokens.push(Token::QueryType(String::from("cookie")));
                 tokens.append(&mut space0.tokenize());
                 tokens.push(Token::CodeDelimiter("\"".to_string()));
                 tokens.append(&mut expr.tokenize());
                 tokens.push(Token::CodeDelimiter("\"".to_string()));
             }
-            QueryValue::Body => tokens.push(Token::QueryType(String::from("body"))),
             QueryValue::Xpath { space0, expr } => {
-                tokens.push(Token::QueryType(String::from("xpath")));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut expr.tokenize());
             }
             QueryValue::Jsonpath { space0, expr } => {
-                tokens.push(Token::QueryType(String::from("jsonpath")));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut expr.tokenize());
             }
             QueryValue::Regex { space0, value } => {
-                tokens.push(Token::QueryType(String::from("regex")));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             QueryValue::Variable { space0, name } => {
-                tokens.push(Token::QueryType(String::from("variable")));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut name.tokenize());
             }
-            QueryValue::Duration => tokens.push(Token::QueryType(String::from("duration"))),
-            QueryValue::Bytes => tokens.push(Token::QueryType(String::from("bytes"))),
-            QueryValue::Sha256 => tokens.push(Token::QueryType(String::from("sha256"))),
-            QueryValue::Md5 => tokens.push(Token::QueryType(String::from("md5"))),
             QueryValue::Certificate {
                 space0,
                 attribute_name: field,
             } => {
-                tokens.push(Token::QueryType(String::from("certificate")));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut field.tokenize());
             }
+            _ => {}
         }
         tokens
     }
@@ -500,16 +494,9 @@ impl Tokenizable for CookieAttribute {
 
 impl Tokenizable for CertificateAttributeName {
     fn tokenize(&self) -> Vec<Token> {
-        let value = match self {
-            CertificateAttributeName::Subject => "Subject",
-            CertificateAttributeName::Issuer => "Issuer",
-            CertificateAttributeName::StartDate => "Start-Date",
-            CertificateAttributeName::ExpireDate => "Expire-Date",
-            CertificateAttributeName::SerialNumber => "Serial-Number",
-        };
         vec![
             Token::StringDelimiter("\"".to_string()),
-            Token::String(value.to_string()),
+            Token::String(self.identifier().to_string()),
             Token::StringDelimiter("\"".to_string()),
         ]
     }
@@ -536,92 +523,98 @@ impl Tokenizable for PredicateFunc {
 impl Tokenizable for PredicateFuncValue {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
+        let name = self.identifier().to_string();
         match self {
             PredicateFuncValue::Equal { space0, value, .. } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::NotEqual { space0, value, .. } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::GreaterThan { space0, value, .. } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::GreaterThanOrEqual { space0, value, .. } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::LessThan { space0, value, .. } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::LessThanOrEqual { space0, value, .. } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::StartWith { space0, value } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::EndWith { space0, value } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::Contain { space0, value } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::Include { space0, value } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
             PredicateFuncValue::Match { space0, value } => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
             }
-
             PredicateFuncValue::IsInteger => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsFloat => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsBoolean => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsString => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsCollection => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsDate => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsIsoDate => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::Exist => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsEmpty => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
             }
             PredicateFuncValue::IsNumber => {
-                tokens.push(Token::PredicateType(self.name()));
+                tokens.push(Token::PredicateType(name));
+            }
+            PredicateFuncValue::IsIpv4 => {
+                tokens.push(Token::PredicateType(name));
+            }
+            PredicateFuncValue::IsIpv6 => {
+                tokens.push(Token::PredicateType(name));
             }
         }
         tokens
@@ -635,7 +628,7 @@ impl Tokenizable for PredicateValue {
             PredicateValue::MultilineString(value) => value.tokenize(),
             PredicateValue::Bool(value) => vec![Token::Boolean(value.to_string())],
             PredicateValue::Null => vec![Token::Keyword("null".to_string())],
-            PredicateValue::Number(value) => vec![Token::Number(value.to_string())],
+            PredicateValue::Number(value) => vec![Token::Number(value.to_source().to_string())],
             PredicateValue::File(value) => value.tokenize(),
             PredicateValue::Hex(value) => vec![Token::String(value.to_string())],
             PredicateValue::Base64(value) => value.tokenize(),
@@ -655,23 +648,13 @@ impl Tokenizable for MultilineString {
             }
             tokens.append(&mut attribute.tokenize());
         }
-        match self {
-            MultilineString {
-                kind: MultilineStringKind::Text(text),
-                ..
-            }
-            | MultilineString {
-                kind: MultilineStringKind::Json(text),
-                ..
-            }
-            | MultilineString {
-                kind: MultilineStringKind::Xml(text),
-                ..
-            } => tokens.append(&mut text.tokenize()),
-            MultilineString {
-                kind: MultilineStringKind::GraphQl(graphql),
-                ..
-            } => tokens.append(&mut graphql.tokenize()),
+        tokens.append(&mut self.space.tokenize());
+        tokens.append(&mut self.newline.tokenize());
+        match &self.kind {
+            MultilineStringKind::Text(value)
+            | MultilineStringKind::Json(value)
+            | MultilineStringKind::Xml(value) => tokens.append(&mut value.tokenize()),
+            MultilineStringKind::GraphQl(graphql) => tokens.append(&mut graphql.tokenize()),
         }
         tokens.push(Token::StringDelimiter("```".to_string()));
         tokens
@@ -687,21 +670,9 @@ impl Tokenizable for MultilineStringAttribute {
     }
 }
 
-impl Tokenizable for Text {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        tokens.append(&mut self.space.tokenize());
-        tokens.append(&mut self.newline.tokenize());
-        tokens.append(&mut self.value.tokenize());
-        tokens
-    }
-}
-
 impl Tokenizable for GraphQl {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
-        tokens.append(&mut self.space.tokenize());
-        tokens.append(&mut self.newline.tokenize());
         tokens.append(&mut self.value.tokenize());
         if let Some(vars) = &self.variables {
             tokens.append(&mut vars.tokenize());
@@ -721,25 +692,6 @@ impl Tokenizable for GraphQlVariables {
     }
 }
 
-impl Tokenizable for EncodedString {
-    fn tokenize(&self) -> Vec<Token> {
-        let mut tokens: Vec<Token> = vec![];
-        if self.quotes {
-            tokens.push(Token::StringDelimiter(
-                if self.quotes { "\"" } else { "" }.to_string(),
-            ));
-        }
-        tokens.push(Token::String(self.encoded.clone()));
-
-        if self.quotes {
-            tokens.push(Token::StringDelimiter(
-                if self.quotes { "\"" } else { "" }.to_string(),
-            ));
-        }
-        tokens
-    }
-}
-
 impl Tokenizable for Template {
     fn tokenize(&self) -> Vec<Token> {
         let mut tokens: Vec<Token> = vec![];
@@ -759,8 +711,8 @@ impl Tokenizable for Template {
 impl Tokenizable for TemplateElement {
     fn tokenize(&self) -> Vec<Token> {
         match self {
-            TemplateElement::String { encoded, .. } => {
-                vec![Token::String(encoded.to_string())]
+            TemplateElement::String { source, .. } => {
+                vec![Token::String(source.to_string())]
             }
             TemplateElement::Placeholder(value) => {
                 let mut tokens: Vec<Token> = vec![];
@@ -930,7 +882,7 @@ impl Tokenizable for EntryOption {
                 .collect(),
         );
         tokens.append(&mut self.space0.tokenize());
-        tokens.push(Token::String(self.kind.name().to_string()));
+        tokens.push(Token::String(self.kind.identifier().to_string()));
         tokens.append(&mut self.space1.tokenize());
         tokens.push(Token::Colon(String::from(":")));
         tokens.append(&mut self.space2.tokenize());
@@ -953,6 +905,7 @@ impl Tokenizable for OptionKind {
             OptionKind::Delay(value) => value.tokenize(),
             OptionKind::FollowLocation(value) => value.tokenize(),
             OptionKind::FollowLocationTrusted(value) => value.tokenize(),
+            OptionKind::Header(value) => value.tokenize(),
             OptionKind::Http10(value) => value.tokenize(),
             OptionKind::Http11(value) => value.tokenize(),
             OptionKind::Http2(value) => value.tokenize(),
@@ -994,12 +947,24 @@ impl Tokenizable for BooleanOption {
 impl Tokenizable for NaturalOption {
     fn tokenize(&self) -> Vec<Token> {
         match self {
-            NaturalOption::Literal(value) => vec![Token::Number(value.to_string())],
+            NaturalOption::Literal(value) => value.tokenize(),
             NaturalOption::Placeholder(expr) => expr.tokenize(),
         }
     }
 }
 
+impl Tokenizable for U64 {
+    fn tokenize(&self) -> Vec<Token> {
+        vec![Token::Number(self.to_source().to_string())]
+    }
+}
+
+impl Tokenizable for I64 {
+    fn tokenize(&self) -> Vec<Token> {
+        vec![Token::Number(self.to_source().to_string())]
+    }
+}
+
 impl Tokenizable for CountOption {
     fn tokenize(&self) -> Vec<Token> {
         match self {
@@ -1029,7 +994,7 @@ impl Tokenizable for DurationOption {
 
 impl Tokenizable for Duration {
     fn tokenize(&self) -> Vec<Token> {
-        let mut tokens = vec![Token::Number(self.value.to_string())];
+        let mut tokens = vec![Token::Number(self.value.to_source().to_string())];
         if let Some(unit) = self.unit {
             tokens.push(Token::Unit(unit.to_string()));
         }
@@ -1053,7 +1018,7 @@ impl Tokenizable for VariableValue {
         match self {
             VariableValue::Null => vec![Token::Keyword("null".to_string())],
             VariableValue::Bool(v) => vec![Token::Boolean(v.to_string())],
-            VariableValue::Number(v) => vec![Token::Number(v.to_string())],
+            VariableValue::Number(v) => vec![Token::Number(v.to_source().to_string())],
             VariableValue::String(v) => v.tokenize(),
         }
     }
@@ -1061,43 +1026,27 @@ impl Tokenizable for VariableValue {
 
 impl Tokenizable for Filter {
     fn tokenize(&self) -> Vec<Token> {
-        match self.value.clone() {
-            FilterValue::Count => vec![Token::FilterType(String::from("count"))],
-            FilterValue::DaysAfterNow => vec![Token::FilterType(String::from("daysAfterNow"))],
-            FilterValue::DaysBeforeNow => vec![Token::FilterType(String::from("daysBeforeNow"))],
+        let mut tokens = vec![Token::FilterType(self.value.identifier().to_string())];
+        match &self.value {
             FilterValue::Decode { space0, encoding } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("decode"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut encoding.tokenize());
-                tokens
             }
             FilterValue::Format { space0, fmt } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("format"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut fmt.tokenize());
-                tokens
-            }
-            FilterValue::HtmlEscape => vec![Token::FilterType(String::from("htmlEscape"))],
-            FilterValue::HtmlUnescape => {
-                vec![Token::FilterType(String::from("htmlUnescape"))]
             }
             FilterValue::JsonPath { space0, expr } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("jsonpath"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut expr.tokenize());
-                tokens
             }
             FilterValue::Nth { space0, n } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("nth"))];
                 tokens.append(&mut space0.tokenize());
-                tokens.push(Token::Number(n.to_string()));
-                tokens
+                tokens.push(Token::Number(n.to_source().to_string()));
             }
             FilterValue::Regex { space0, value } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("regex"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut value.tokenize());
-                tokens
             }
             FilterValue::Replace {
                 space0,
@@ -1105,35 +1054,25 @@ impl Tokenizable for Filter {
                 space1,
                 new_value,
             } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("replace"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut old_value.tokenize());
                 tokens.append(&mut space1.tokenize());
                 tokens.append(&mut new_value.tokenize());
-                tokens
             }
-            FilterValue::UrlEncode => vec![Token::FilterType(String::from("urlEncode"))],
-            FilterValue::UrlDecode => vec![Token::FilterType(String::from("urlDecode"))],
             FilterValue::Split { space0, sep } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("split"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut sep.tokenize());
-                tokens
             }
             FilterValue::ToDate { space0, fmt } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("toDate"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut fmt.tokenize());
-                tokens
             }
-            FilterValue::ToFloat => vec![Token::FilterType(String::from("toFloat"))],
-            FilterValue::ToInt => vec![Token::FilterType(String::from("toInt"))],
             FilterValue::XPath { space0, expr } => {
-                let mut tokens: Vec<Token> = vec![Token::FilterType(String::from("xpath"))];
                 tokens.append(&mut space0.tokenize());
                 tokens.append(&mut expr.tokenize());
-                tokens
             }
+            _ => {}
         }
+        tokens
     }
 }
