@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,23 +15,17 @@
  * limitations under the License.
  *
  */
-use crate::linter::{LinterError, LinterErrorKind};
 use hurl_core::ast::{
     Assert, Base64, Body, Bytes, Capture, Comment, Cookie, CookieAttribute, CookieAttributeName,
     CookiePath, DurationOption, Entry, EntryOption, File, FileParam, Filter, FilterValue, GraphQl,
     Hex, HurlFile, KeyValue, LineTerminator, MultilineString, MultilineStringAttribute,
     MultilineStringKind, MultipartParam, OptionKind, Predicate, PredicateFunc, PredicateFuncValue,
     PredicateValue, Query, QueryValue, RegexValue, Request, Response, Section, SectionValue,
-    SourceInfo, Template, Text, VariableDefinition, Whitespace,
+    SourceInfo, Template, VariableDefinition, Whitespace,
 };
 use hurl_core::reader::Pos;
 use hurl_core::typing::{Duration, DurationUnit};
 
-/// Returns lint errors for the `hurl_file`.
-pub fn check_hurl_file(hurl_file: &HurlFile) -> Vec<LinterError> {
-    hurl_file.entries.iter().flat_map(check_entry).collect()
-}
-
 /// Returns a new linted instance from this `hurl_file`.
 pub fn lint_hurl_file(hurl_file: &HurlFile) -> HurlFile {
     HurlFile {
@@ -40,42 +34,12 @@ pub fn lint_hurl_file(hurl_file: &HurlFile) -> HurlFile {
     }
 }
 
-fn check_entry(entry: &Entry) -> Vec<LinterError> {
-    let mut errors = vec![];
-    errors.append(&mut check_request(&entry.request));
-    if let Some(response) = &entry.response {
-        errors.append(&mut check_response(response));
-    }
-    errors
-}
-
 fn lint_entry(entry: &Entry) -> Entry {
     let request = lint_request(&entry.request);
     let response = entry.response.as_ref().map(lint_response);
     Entry { request, response }
 }
 
-fn check_request(request: &Request) -> Vec<LinterError> {
-    let mut errors = vec![];
-    if !request.space0.value.is_empty() {
-        errors.push(LinterError {
-            source_info: request.space0.source_info,
-            kind: LinterErrorKind::UnnecessarySpace,
-        });
-    }
-    if request.space1.value != " " {
-        errors.push(LinterError {
-            source_info: request.space1.source_info,
-            kind: LinterErrorKind::OneSpace,
-        });
-    }
-    for error in check_line_terminator(&request.line_terminator0) {
-        errors.push(error);
-    }
-    errors.extend(request.sections.iter().flat_map(check_section));
-    errors
-}
-
 fn lint_request(request: &Request) -> Request {
     let line_terminators = request.line_terminators.clone();
     let space0 = empty_whitespace();
@@ -104,18 +68,6 @@ fn lint_request(request: &Request) -> Request {
     }
 }
 
-fn check_response(response: &Response) -> Vec<LinterError> {
-    let mut errors = vec![];
-    if !response.space0.value.is_empty() {
-        errors.push(LinterError {
-            source_info: response.space0.source_info,
-            kind: LinterErrorKind::UnnecessarySpace,
-        });
-    }
-    errors.extend(response.sections.iter().flat_map(check_section));
-    errors
-}
-
 fn lint_response(response: &Response) -> Response {
     let line_terminators = response.line_terminators.clone();
     let space0 = empty_whitespace();
@@ -142,20 +94,6 @@ fn lint_response(response: &Response) -> Response {
     }
 }
 
-fn check_section(section: &Section) -> Vec<LinterError> {
-    let mut errors = vec![];
-    if !section.space0.value.is_empty() {
-        errors.push(LinterError {
-            source_info: section.space0.source_info,
-            kind: LinterErrorKind::UnnecessarySpace,
-        });
-    }
-    for error in check_line_terminator(&section.line_terminator0) {
-        errors.push(error);
-    }
-    errors
-}
-
 fn lint_section(section: &Section) -> Section {
     let line_terminators = section.line_terminators.clone();
     let line_terminator0 = section.line_terminator0.clone();
@@ -237,6 +175,11 @@ fn lint_capture(capture: &Capture) -> Capture {
         .iter()
         .map(|(_, f)| (one_whitespace(), lint_filter(f)))
         .collect();
+    let space3 = if capture.redact {
+        one_whitespace()
+    } else {
+        empty_whitespace()
+    };
     Capture {
         line_terminators: capture.line_terminators.clone(),
         space0: empty_whitespace(),
@@ -245,6 +188,8 @@ fn lint_capture(capture: &Capture) -> Capture {
         space2: one_whitespace(),
         query: lint_query(&capture.query),
         filters,
+        space3,
+        redact: capture.redact,
         line_terminator0: lint_line_terminator(&capture.line_terminator0),
     }
 }
@@ -259,6 +204,7 @@ fn lint_query(query: &Query) -> Query {
 fn lint_query_value(query_value: &QueryValue) -> QueryValue {
     match query_value {
         QueryValue::Status => QueryValue::Status,
+        QueryValue::Version => QueryValue::Version,
         QueryValue::Url => QueryValue::Url,
         QueryValue::Header { name, .. } => QueryValue::Header {
             name: name.clone(),
@@ -305,6 +251,7 @@ fn lint_query_value(query_value: &QueryValue) -> QueryValue {
             attribute_name: *field,
             space0: one_whitespace(),
         },
+        QueryValue::Ip => QueryValue::Ip,
     }
 }
 
@@ -363,34 +310,28 @@ fn lint_predicate_func_value(predicate_func_value: &PredicateFuncValue) -> Predi
         PredicateFuncValue::Equal { value, .. } => PredicateFuncValue::Equal {
             space0: one_whitespace(),
             value: lint_predicate_value(value),
-            operator: true,
         },
         PredicateFuncValue::NotEqual { value, .. } => PredicateFuncValue::NotEqual {
             space0: one_whitespace(),
             value: lint_predicate_value(value),
-            operator: true,
         },
         PredicateFuncValue::GreaterThan { value, .. } => PredicateFuncValue::GreaterThan {
             space0: one_whitespace(),
             value: lint_predicate_value(value),
-            operator: true,
         },
         PredicateFuncValue::GreaterThanOrEqual { value, .. } => {
             PredicateFuncValue::GreaterThanOrEqual {
                 space0: one_whitespace(),
                 value: lint_predicate_value(value),
-                operator: true,
             }
         }
         PredicateFuncValue::LessThan { value, .. } => PredicateFuncValue::LessThan {
             space0: one_whitespace(),
             value: lint_predicate_value(value),
-            operator: true,
         },
         PredicateFuncValue::LessThanOrEqual { value, .. } => PredicateFuncValue::LessThanOrEqual {
             space0: one_whitespace(),
             value: lint_predicate_value(value),
-            operator: true,
         },
         PredicateFuncValue::Contain { value, .. } => PredicateFuncValue::Contain {
             space0: one_whitespace(),
@@ -424,6 +365,8 @@ fn lint_predicate_func_value(predicate_func_value: &PredicateFuncValue) -> Predi
         PredicateFuncValue::Exist => PredicateFuncValue::Exist,
         PredicateFuncValue::IsEmpty => PredicateFuncValue::IsEmpty,
         PredicateFuncValue::IsNumber => PredicateFuncValue::IsNumber,
+        PredicateFuncValue::IsIpv4 => PredicateFuncValue::IsIpv4,
+        PredicateFuncValue::IsIpv6 => PredicateFuncValue::IsIpv6,
     }
 }
 
@@ -445,34 +388,48 @@ fn lint_predicate_value(predicate_value: &PredicateValue) -> PredicateValue {
 }
 
 fn lint_multiline_string(multiline_string: &MultilineString) -> MultilineString {
+    let space = empty_whitespace();
+    let newline = multiline_string.newline.clone();
     match multiline_string {
         MultilineString {
+            attributes,
             kind: MultilineStringKind::Text(value),
-            attributes,
+            ..
         } => MultilineString {
-            kind: MultilineStringKind::Text(lint_text(value)),
             attributes: lint_multiline_string_attributes(attributes),
+            space,
+            newline,
+            kind: MultilineStringKind::Text(lint_template(value)),
         },
         MultilineString {
+            attributes,
             kind: MultilineStringKind::Json(value),
-            attributes,
+            ..
         } => MultilineString {
-            kind: MultilineStringKind::Json(lint_text(value)),
             attributes: lint_multiline_string_attributes(attributes),
+            space,
+            newline,
+            kind: MultilineStringKind::Json(lint_template(value)),
         },
         MultilineString {
+            attributes,
             kind: MultilineStringKind::Xml(value),
-            attributes,
+            ..
         } => MultilineString {
-            kind: MultilineStringKind::Xml(lint_text(value)),
             attributes: lint_multiline_string_attributes(attributes),
+            space,
+            newline,
+            kind: MultilineStringKind::Xml(lint_template(value)),
         },
         MultilineString {
-            kind: MultilineStringKind::GraphQl(value),
             attributes,
+            kind: MultilineStringKind::GraphQl(value),
+            ..
         } => MultilineString {
-            kind: MultilineStringKind::GraphQl(lint_graphql(value)),
             attributes: lint_multiline_string_attributes(attributes),
+            space,
+            newline,
+            kind: MultilineStringKind::GraphQl(lint_graphql(value)),
         },
     }
 }
@@ -483,28 +440,10 @@ fn lint_multiline_string_attributes(
     attributes.to_vec()
 }
 
-fn lint_text(text: &Text) -> Text {
-    let space = empty_whitespace();
-    let newline = text.newline.clone();
-    let value = lint_template(&text.value);
-    Text {
-        space,
-        newline,
-        value,
-    }
-}
-
 fn lint_graphql(graphql: &GraphQl) -> GraphQl {
-    let space = empty_whitespace();
-    let newline = graphql.newline.clone();
     let value = lint_template(&graphql.value);
     let variables = graphql.variables.clone();
-    GraphQl {
-        space,
-        newline,
-        value,
-        variables,
-    }
+    GraphQl { value, variables }
 }
 
 fn lint_cookie(cookie: &Cookie) -> Cookie {
@@ -540,7 +479,7 @@ fn lint_base64(base64: &Base64) -> Base64 {
     Base64 {
         space0: empty_whitespace(),
         value: base64.value.clone(),
-        encoded: base64.encoded.clone(),
+        source: base64.source.clone(),
         space1: empty_whitespace(),
     }
 }
@@ -549,7 +488,7 @@ fn lint_hex(hex: &Hex) -> Hex {
     Hex {
         space0: empty_whitespace(),
         value: hex.value.clone(),
-        encoded: hex.encoded.clone(),
+        source: hex.source.clone(),
         space1: empty_whitespace(),
     }
 }
@@ -620,22 +559,6 @@ fn one_whitespace() -> Whitespace {
     }
 }
 
-fn check_line_terminator(line_terminator: &LineTerminator) -> Vec<LinterError> {
-    let mut errors = vec![];
-    match &line_terminator.comment {
-        Some(_) => {}
-        None => {
-            if !line_terminator.space0.value.is_empty() {
-                errors.push(LinterError {
-                    source_info: line_terminator.space0.source_info,
-                    kind: LinterErrorKind::UnnecessarySpace,
-                });
-            }
-        }
-    }
-    errors
-}
-
 fn lint_line_terminator(line_terminator: &LineTerminator) -> LineTerminator {
     let space0 = match line_terminator.comment {
         None => empty_whitespace(),
@@ -712,7 +635,7 @@ fn lint_duration_option(
 }
 
 fn lint_duration(duration: &Duration, default_unit: DurationUnit) -> Duration {
-    let value = duration.value;
+    let value = duration.value.clone();
     let unit = Some(duration.unit.unwrap_or(default_unit));
     Duration { value, unit }
 }
@@ -756,7 +679,6 @@ mod tests {
             entries: vec![],
             line_terminators: vec![],
         };
-        assert_eq!(check_hurl_file(&hurl_file), vec![]);
         assert_eq!(lint_hurl_file(&hurl_file), hurl_file_linted);
     }
 
@@ -770,7 +692,6 @@ mod tests {
             entries: vec![],
             line_terminators: vec![],
         };
-        assert_eq!(check_hurl_file(&entry), vec![]);
         assert_eq!(lint_hurl_file(&entry), entry_linted);
     }
 }
